/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "XPathResult.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomXPathResult.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomXPathResult.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomXPathResult.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::XPathResult* obj)
{
    return WebCore::cacheDOMObject<WebCore::XPathResult, GDOMXPathResult>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMXPathResult::GDOMXPathResult(PassRefPtr<WebCore::XPathResult> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMXPathResult::~GDOMXPathResult()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

GdomNode *
gdom_x_path_result_iterate_next (GdomXPathResult *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XPathResult * item = (gdom_x_path_result_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->iterateNext(ec));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_x_path_result_snapshot_item (GdomXPathResult *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XPathResult * item = (gdom_x_path_result_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->snapshotItem(index, ec));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "XPathResult.h"
#include <gdom/GdomXPathResult.h>
#include <gdom/PrivateGdomXPathResult.h>
#include <gdom/PrivateIntGdomXPathResult.h>
#include <gdom/GdomXPathResultprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomXPathResult, gdom_x_path_result, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomXPathResultPrivate {
    class WebCore::XPathResult *item;
};

#define GDOM_X_PATH_RESULT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_X_PATH_RESULT, GdomXPathResultPrivate))
*/
G_END_DECLS
WebCore::XPathResult *
gdom_x_path_result_get_private(GdomXPathResult *request)
{
    GDOMXPathResult *res;
    res = static_cast<GDOMXPathResult*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::XPathResult*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_RESULT_TYPE,
    PROP_NUMBER_VALUE,
    PROP_STRING_VALUE,
    PROP_BOOLEAN_VALUE,
    PROP_SINGLE_NODE_VALUE,
    PROP_INVALID_ITERATOR_STATE,
    PROP_SNAPSHOT_LENGTH,
};


static void gdom_x_path_result_finalize(GObject* object)
{
    /*GdomXPathResult* request = GDOM_X_PATH_RESULT(object);*/
    /*GdomXPathResultPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_x_path_result_parent_class)->finalize(object);
}

static void gdom_x_path_result__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomXPathResult *self = GDOM_X_PATH_RESULT(object);
    WebCore::XPathResult * gself = (gdom_x_path_result_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_x_path_result__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomXPathResult *self = GDOM_X_PATH_RESULT(object);
    WebCore::XPathResult * gself = (gdom_x_path_result_get_private(self));
    switch (prop_id) {
    case PROP_RESULT_TYPE:
    {
         g_value_set_uint (value, gself->resultType());
         break;
    }
    case PROP_NUMBER_VALUE:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_double (value, gself->numberValue(ec));
         break;
    }
    case PROP_STRING_VALUE:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_string (value, _G_GET_UTF8(gself->stringValue(ec)));
         break;
    }
    case PROP_BOOLEAN_VALUE:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_boolean (value, gself->booleanValue(ec));
         break;
    }
    case PROP_SINGLE_NODE_VALUE:
    {
         WebCore::ExceptionCode ec = 0;
         PassRefPtr<WebCore::Node> ptr = gself->singleNodeValue(ec);
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_INVALID_ITERATOR_STATE:
    {
         g_value_set_boolean (value, gself->invalidIteratorState());
         break;
    }
    case PROP_SNAPSHOT_LENGTH:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_ulong (value, gself->snapshotLength(ec));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_x_path_result_class_init(GdomXPathResultClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_x_path_result_finalize;
    g_klass->set_property = gdom_x_path_result__set_property_fn;
    g_klass->get_property = gdom_x_path_result__get_property_fn;

    g_object_class_install_property (g_klass, PROP_RESULT_TYPE,
            g_param_spec_uint ("result_type", /* name */
                "x_path_result_result_type", /* short description */
                "read-only  gushort XPathResult.result_type", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NUMBER_VALUE,
            g_param_spec_double ("number_value", /* name */
                "x_path_result_number_value", /* short description */
                "read-only  gdouble XPathResult.number_value", /* longer - could do with some extra doc stuff here */
                 G_MINDOUBLE,        /* min */
                G_MAXDOUBLE, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_STRING_VALUE,
            g_param_spec_string ("string_value", /* name */
                "x_path_result_string_value", /* short description */
                "read-only  gchar * XPathResult.string_value", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_BOOLEAN_VALUE,
            g_param_spec_boolean ("boolean_value", /* name */
                "x_path_result_boolean_value", /* short description */
                "read-only  gboolean XPathResult.boolean_value", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SINGLE_NODE_VALUE,
            g_param_spec_object ("single_node_value", /* name */
                "x_path_result_single_node_value", /* short description */
                "read-only  GdomNode * XPathResult.single_node_value", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_INVALID_ITERATOR_STATE,
            g_param_spec_boolean ("invalid_iterator_state", /* name */
                "x_path_result_invalid_iterator_state", /* short description */
                "read-only  gboolean XPathResult.invalid_iterator_state", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SNAPSHOT_LENGTH,
            g_param_spec_ulong ("snapshot_length", /* name */
                "x_path_result_snapshot_length", /* short description */
                "read-only  gulong XPathResult.snapshot_length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomXPathResultPrivate));*/
}

static void gdom_x_path_result_init(GdomXPathResult* request)
{
    /*GdomXPathResultPrivate* priv = GDOM_X_PATH_RESULT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

